@@ -19,6 +19,10 @@
 */
 #include "sparrowCore.h"
 #include "sparrowMath.h"
+#include "sparrowPrimitives.h"
+#include <SDL.h>
+#include <SDL_image.h>
+#include <SDL_ttf.h>
 
 int spWindowX;
 int spWindowY;
@@ -29,13 +33,21 @@ SDL_Joystick *spJoy = NULL;
 char spDone;
 int spFPS;
 TspInput spInput;
+int debug_time;
 
 PREFIX void spInitCore(void)
 {
+  debug_time = 0;
   TTF_Init();
   #ifdef PANDORA
     spWindowX = 800;
     spWindowY = 480;
+  #elif defined MAEMO5
+    spWindowX = 800;
+    spWindowY = 480;
+  #elif defined MAEMO6
+    spWindowX = 854;
+    spWindowY = 480;
   #else
     spWindowX = 320;
     spWindowY = 240;
@@ -54,9 +66,21 @@ PREFIX void spInitCore(void)
     spInput.button[i]=0;
   spInput.axis[0]=0;
   spInput.axis[1]=0;
+  spInitPrimitives();
 }
 
-void spResizeWindow(int x,int y)
+PREFIX void spPrintDebug(char* text)
+{
+  Sint32 time = SDL_GetTicks();
+  int time_diff = time-debug_time;
+  if (time_diff > 100)
+    printf("%05i (%3i LONG!): %s\n",time,time_diff,text);
+  else
+    printf("%05i (%3i): %s\n",time,time_diff,text);
+  debug_time = time;
+}
+
+inline void spResizeWindow(int x,int y)
 {
    #ifdef GP2X
     spScreen=SDL_SetVideoMode(x,y,16,SDL_HWSURFACE);
@@ -74,6 +98,9 @@ void spResizeWindow(int x,int y)
   #elif defined PANDORA
     spScreen=NULL;
     spWindow=SDL_SetVideoMode(x,y,16,SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+  #elif defined MAEMO
+    spScreen=NULL;
+    spWindow=SDL_SetVideoMode(x,y,16,SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN | SDL_HWPALETTE);
   #else
     /*x=800;
     y=480;*/
@@ -87,6 +114,7 @@ void spResizeWindow(int x,int y)
     spWindowX = x ;
   spWindowY = y;
   spZoom=spMin((spWindowX<<SP_ACCURACY)/320,(spWindowY<<SP_ACCURACY)/240);  //Bei 320x240 == 1
+  SDL_ShowCursor(SDL_DISABLE);
 }
 
 PREFIX SDL_Surface* spCreateWindow(void)
@@ -100,17 +128,27 @@ PREFIX SDL_Surface* spGetWindowSurface(void)
   return spWindow;
 }
 
-void spHandleEvent(void)
+inline int spHandleEvent(void)
 {
+  int result = 0;
   #ifdef PANDORA
     /*int s = pnd_evdev_dpad_state ( pnd_evdev_dpads );
     if ( s & pnd_evdev_pandora )
       spDone=1;*/
   #endif
   char new_axis = 0;
+  #ifdef CORE_DEBUG
+    int counter = 0;
+  #endif
   SDL_Event event; 
   while (SDL_PollEvent(&event)==1)
   {
+    #ifdef CORE_DEBUG
+      counter++;
+      char buffer[32];
+      sprintf(buffer,"    Fetching Event %i\n",counter);
+      spPrintDebug(buffer);
+    #endif
     switch (event.type)
     {
       case SDL_JOYBUTTONDOWN:
@@ -369,9 +407,11 @@ void spHandleEvent(void)
         break;
       case SDL_VIDEORESIZE:
         spResizeWindow(event.resize.w,event.resize.h);
+        result = 1;
         break;
     }
   }
+  return result;
 }
 
 /*PREFIX signed char spGetAxis(int axis)
@@ -468,7 +508,7 @@ void spHandleEvent(void)
   #endif
 }*/
 
-void spUpdateAxis(int axis)
+inline void spUpdateAxis(int axis)
 {
   #ifdef GP2X
     if (axis==0)
@@ -502,7 +542,7 @@ Uint32 oldticks;
 Uint32 olderticks;
 Uint32 newticks;
 
-PREFIX int spLoop(void (*spDraw)(void),int (*spCalc)(Uint32 steps),Uint32 minwait)
+PREFIX int spLoop(void (*spDraw)(void),int (*spCalc)(Uint32 steps),Uint32 minwait,void (*spResize)(Uint16 w,Uint16 h))
 {
   Uint32 bigsteps=0;
   Uint32 frames=0;
@@ -512,39 +552,64 @@ PREFIX int spLoop(void (*spDraw)(void),int (*spCalc)(Uint32 steps),Uint32 minwai
   oldticks=olderticks;
   newticks=olderticks;
   while(back==0 && !spDone ) {
+      #ifdef CORE_DEBUG
+        spPrintDebug("Start mainloop");
+      #endif
       newticks=SDL_GetTicks();
-      spHandleEvent();
+      if (spHandleEvent() && spResize)
+      {
+        spResize(spWindowX,spWindowY);
+        #ifdef CORE_DEBUG
+          spPrintDebug("  Did events and resize");
+        #endif
+      }
+      #ifdef CORE_DEBUG
+        else
+          spPrintDebug("  Did events");
+      #endif
       spUpdateAxis(0);
       spUpdateAxis(1);
+      #ifdef CORE_DEBUG
+        spPrintDebug("  Did axis update");
+      #endif
       if (newticks-oldticks > 0)
       {
         back = spCalc(newticks-oldticks);
+        #ifdef CORE_DEBUG
+          spPrintDebug("  Did calc");
+        #endif
         oldticks = newticks;
       }
       steps+=newticks-olderticks;
-      if (steps>minwait)
+      if (steps>=minwait)
       {
         spDraw();
+        #ifdef CORE_DEBUG
+          spPrintDebug("  Did draw");
+        #endif
         frames++;
         bigsteps=bigsteps+steps;
         while (bigsteps>=1000)
         {
           bigsteps-=1000;
           spFPS=frames;
-          printf("FPS: %i\n",spFPS);
+          //printf("FPS: %i\n",spFPS);
           frames=0;
         }
+        //if (steps-minwait > 0)
+        //  SDL_Delay(steps-minwait);
         steps=0;
         olderticks = newticks;
       }
-      if (newticks == oldticks)
-        SDL_Delay(1);
   }
   return back;  
 }
 
 PREFIX void spFlip(void)
 {
+  #ifdef CORE_DEBUG
+    spPrintDebug("    Flip in");
+  #endif
   //The Flip
   #ifdef GP2X
     SDL_BlitSurface(spWindow, NULL, spScreen, NULL);
@@ -556,6 +621,10 @@ PREFIX void spFlip(void)
     SDL_Flip(spWindow);
   #else //PC, Dingoo and Pandora
     SDL_Flip(spWindow);
+    //SDL_UpdateRect(spWindow, 0, 0, 0, 0);
+  #endif
+  #ifdef CORE_DEBUG
+    spPrintDebug("    Flip out");
   #endif
 }
 
@@ -571,3 +640,52 @@ PREFIX void spQuitCore(void)
   #endif
   SDL_Quit();
 }
+
+PREFIX int spGetFPS(void)
+{
+  return spFPS;
+}
+
+PREFIX Sint32 spGetSizeFactor(void)
+{
+  return spZoom;
+}
+
+PREFIX SDL_Surface* spLoadSurface(char* name)
+{
+  SDL_Surface* surface = IMG_Load(name);
+  /*printf("%s: %i\n",name,surface->format->BitsPerPixel);
+  if (surface->format->BitsPerPixel==32) //removing alpha
+  {
+    SDL_LockSurface(surface);
+    Uint8* pixel = (Uint8*)(surface->pixels);
+    int x,y;
+    for (x = 0; x < surface->w; x++)
+      for (y = 0; y < surface->h; y++)
+      {
+        if (pixel[(x+y*surface->w)*4+3] > 128)
+        {
+          pixel[(x+y*surface->w)*4+3] = 255;
+          if (pixel[(x+y*surface->w)*4+0] == 255 &&
+              pixel[(x+y*surface->w)*4+1] == 0   &&
+              pixel[(x+y*surface->w)*4+2] == 255 ) //removing pink
+          {
+            pixel[(x+y*surface->w)*4+0] = 247;
+            pixel[(x+y*surface->w)*4+2] = 247;
+          }
+        }
+        else
+        {
+          pixel[(x+y*surface->w)*4+0] = 255;
+          pixel[(x+y*surface->w)*4+1] = 0;
+          pixel[(x+y*surface->w)*4+2] = 255;
+          pixel[(x+y*surface->w)*4+3] = 255;
+        }       
+      }
+    
+    SDL_UnlockSurface(surface);
+  }*/
+  SDL_Surface* result = SDL_DisplayFormat(surface);
+  SDL_FreeSurface(surface);
+  return result;
+}
